{# Filename: .templates/layout.html #}
{% extends '!layout.html' %}

{% block relbar1 %}
<div id="logo">
<a href="/"><img src="{{ pathto('_static/quantumcore-small.png',1) }}"/></a>
</div>
    <div class="related">
      <h3>Navigation</h3>
      <ul>
        {%- for rellink in rellinks %}
        <li class="right" {% if loop.first %}style="margin-right: 10px"{% endif %}>
          <a href="{{ pathto(rellink[0]) }}" title="{{ rellink[1]|striptags }}"
             accesskey="{{ rellink[2] }}">{{ rellink[3] }}</a>
          {%- if not loop.first %}{{ reldelim2 }}{% endif %}</li>
        {%- endfor %}
        {%- block rootrellink %}
        <li><a class="homelink" href="{{ pathto('index') }}">{{ shorttitle }}</a>{{ reldelim1 }}</li>
        {%- endblock %}
        {%- for parent in parents %}
          <li><a href="{{ parent.link|e }}" accesskey="U">{{ parent.title }}</a>{{ reldelim1 }}</li>
        {%- endfor %}
        {%- block relbaritems %}{% endblock %}
      </ul>
    </div>

{% endblock %}

