<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" 
                      "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
      xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">
<?python
import tg
tg_flash = tg.get_flash()
?>
<head py:match="head" py:attrs="select('@*')">
    <meta content="text/html; charset=UTF-8" http-equiv="content-type" py:replace="''"/>
    <title py:replace="''">Your title goes here</title>
    <meta py:replace="select('*')"/>
    <link rel="stylesheet" type="text/css" media="screen" href="/css/style.css" />
</head>

<body py:match="body" py:attrs="select('@*')">
  ${header()}
  <div id="main_content">
    <div id="status_block" py:if="tg_flash" class="flash" py:content="tg_flash"></div>
    <div py:replace="select('*|text()')"/>
    <!-- End of main_content -->
    <div class="clearingdiv" />
    ${footer()}
  </div>
</body>
</html>
