<html xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">

<py:def function="sidebar_top">
  <div id="sb_top" class="sidebar">
      <h2>Get started with the LDAP plugin</h2>
      <ul class="links">
        <li>
            <a href="http://code.gustavonarea.net/repoze.who.plugins.ldap/"
            >Plugin homepage</a>.
        </li>
        <li>
            <a href="http://static.repoze.org/whodocs/">repoze.who
            homepage</a>.
        </li>
        <li><a href="http://lists.repoze.org/listinfo/repoze-dev">Get help</a>.</li>
      </ul>
  </div>
</py:def>

</html>
