<html xmlns:py="http://genshi.edgewall.org/"
      xmlns:xi="http://www.w3.org/2001/XInclude"
      py:strip="">
<py:def function="footer">
<div id="footer"> 
  <div class="flogo">
    <img src="/images/under_the_hood_blue.png" alt="TurboGears" />
    <p><a href="http://www.turbogears.org/2.0/">Powered by TurboGears 2</a></p>
  </div>
  <div class="foottext">
    <p>TurboGears is a open source front-to-back web development
      framework written in Python. Copyright (c) 2005-2008 </p>
  </div>
</div>
</py:def>    
</html>
